@@ -20,6 +20,7 @@ import oslo_messaging
 from neutron.db.models import l3agent
 from neutron.db.models import servicetype
 from neutron_lib import constants as lib_constants
+from neutron_lib.db import api as db_api
 from neutron_lib.plugins import directory
 
 from neutron_vpnaas.db.vpn import vpn_models
@@ -70,6 +71,7 @@ class IPsecVpnDriverCallBack(object):
             self.driver.name)
         return query
 
+    @db_api.CONTEXT_READER
     def get_vpn_services_on_host(self, context, host=None):
         """Returns the vpnservices on the host."""
         vpnservices = self._get_agent_hosting_vpn_services(
@@ -112,20 +114,20 @@ class BaseIPsecVPNDriver(service_drivers.VpnDriver, metaclass=abc.ABCMeta):
         pass
 
     def create_ipsec_site_connection(self, context, ipsec_site_connection):
-        vpnservice = self.service_plugin._get_vpnservice(
+        router_id = self.service_plugin.get_vpnservice_router_id(
             context, ipsec_site_connection['vpnservice_id'])
-        self.agent_rpc.vpnservice_updated(context, vpnservice['router_id'])
+        self.agent_rpc.vpnservice_updated(context, router_id)
 
     def update_ipsec_site_connection(
         self, context, old_ipsec_site_connection, ipsec_site_connection):
-        vpnservice = self.service_plugin._get_vpnservice(
+        router_id = self.service_plugin.get_vpnservice_router_id(
             context, ipsec_site_connection['vpnservice_id'])
-        self.agent_rpc.vpnservice_updated(context, vpnservice['router_id'])
+        self.agent_rpc.vpnservice_updated(context, router_id)
 
     def delete_ipsec_site_connection(self, context, ipsec_site_connection):
-        vpnservice = self.service_plugin._get_vpnservice(
+        router_id = self.service_plugin.get_vpnservice_router_id(
             context, ipsec_site_connection['vpnservice_id'])
-        self.agent_rpc.vpnservice_updated(context, vpnservice['router_id'])
+        self.agent_rpc.vpnservice_updated(context, router_id)
 
     def create_ikepolicy(self, context, ikepolicy):
         pass
@@ -161,6 +163,7 @@ class BaseIPsecVPNDriver(service_drivers.VpnDriver, metaclass=abc.ABCMeta):
                 v6_ip = addr
         return v4_ip, v6_ip
 
+    @db_api.CONTEXT_WRITER
     def create_vpnservice(self, context, vpnservice_dict):
         """Get the gateway IP(s) and save for later use.
 
